#!/usr/bin/perl
#
# This file is part of Config-Model-LcdProc
#
# This software is Copyright (c) 2016 by Dominique Dumont.
#
# This is free software, licensed under:
#
#   The GNU Lesser General Public License, Version 2.1, February 1999
#

# test headers provided by Dist;:Zilla. Well, provided means
# "ge.ne.ra.ted" but that breaks the test as licensecheck will issue
# "GE.NE.RA.TED FILE"

use strict;
use warnings;

# This script uses all the information available in LCDd.conf to create a model
# for LCDd configuration file
